@@ -2,6 +2,7 @@
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
+from QXApplication import *
 import platform
 import sys
 import re
@@ -16,7 +17,7 @@ class QXAction(QAction):
         kq = QKeySequence.mnemonic(self.text())
         if kq.isEmpty() or platform.system() == 'Windows':
             if self.text().contains('&'):
-                t = self.text().replace('&','')
+                t = self.text().replace('&','').replace('.','')
                 try:
                     kq = QKeySequence(eval('QKeySequence.%s' % t))
                 except:
@@ -24,13 +25,10 @@ class QXAction(QAction):
         self.setShortcut(kq)
         
         if not kq.isEmpty():
-            self.setToolTip("%s (%s)" % (self.text().replace('&',''),kq.toString(QKeySequence.NativeText)))
+            self.setToolTip("%s (%s)" % (self.text().replace('&','').replace('.',''),kq.toString(QKeySequence.NativeText)))
         
-        image_name = "%s.png" % self.text().replace('&','').toLower()
-        dir = QDir(QDir.currentPath());
-        if dir.exists(image_name):
-            self.setIcon(QIcon(image_name))
-        else:
-            image_name = ":" + image_name
-            if dir.exists(image_name):
-                self.setIcon(QIcon(image_name))   
\ No newline at end of file
+        image_name = self.text().replace('&','').replace('.','').toLower()
+        if image_name == 'preferences':
+            image_name = 'configure'
+        self.setIcon(QXApplication.findIcon(image_name,'default',self.setIcon)) 
+                  
\ No newline at end of file
